#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
//#include <fcntl.h>

// use nice portable types
#include "g_types.h"

#include "config.h"
#include "urec.h"
#include "mtop.h"


void generateTopNScores(
	struct urec *users, int32 totalusers, struct userscore *scores, int32 sumofscores, 
	int32 listlen, int32 (*getData)(struct urec *), int (*cmp)(const void *, const void *))
{
	int idx;

	// sort the users by their raw scores
	qsort(users, totalusers, sizeof(struct urec), cmp);

	// generate a score report
	for(int32 i = 0; i < listlen; i++)
	{
		idx = totalusers - i - 1;
		strncpy(scores[i].handle, users[idx].handle, sizeof(scores[i].handle));
		scores[i].score = (float64)getData(&(users[idx]));
		scores[i].percentage = scores[i].score / (float64)sumofscores * 100.0;
	}
}

void updateReport(const char *tfn, const char *rfn, struct userscore *scores, int32 listlen,
				  char hi_ch, char *hi_pi, char lo_ch, char *lo_pi)
{
	char buf[5], output[31];
	int32 rank, i, numbytes;

	// FILE *in is the template file, *out is the report file. 
	FILE *in, *out;

	in = fopen(tfn, "rb");
	out = fopen(rfn, "wb");

	while(fread(&(buf[0]), sizeof(char), 1, in) == 1)
	{
		if(buf[0] == '|')
		{
			// watch out for pipes lurking near the end of the file
			if(fread(&(buf[1]), sizeof(char), 1, in) < 1)
			{
				fwrite(buf, sizeof(char), 1, out);
			}
			else if(buf[1] == 'P' || buf[1] == 'U' || buf[1] == 'N' || buf[1] == 'G')
			{
				// beware the jabberwock
				if((numbytes = fread(&(buf[2]), sizeof(char), 2, in)) < 2)
				{
					fwrite(buf, sizeof(char), 2 + numbytes, out);
				}
				else if(isdigit(buf[2]) && isdigit(buf[3]))
				{
					// the MCI system uses a 1..n number system.  convert to a 0..(n-1) system
					// so the user's rank can be used as an array index.
					// rank = (((buf[2] - '0') * 10) + (buf[3] - '0') + listlen - 1) % listlen;
					rank = atoi(&(buf[2])) - 1;

					// if(rank >= listlen) then there are less users in users.dat than there are slots
					// in the list.  this is a bad thing, compensate for it by printing spaces where
					// the normal output would be to keep from fucking up the report's layout.
					switch(buf[1])
					{
					case 'P':
						// print the score as a percentage of the total scores for the board
						if(rank >= listlen) memset(output, 32, 3);
						else sprintf(output, "%3d", (int)scores[rank].percentage);
						fwrite(output, sizeof(char), 3, out);
						continue;
					case 'U':
						// print the username
						if(rank >= listlen) memset(output, 32, 30);
						else sprintf(output, "%-30s", scores[rank].handle);
						fwrite(output, sizeof(char), 30, out);
						continue;
					case 'N':
						// print the raw score
						if(rank >= listlen) memset(output, 32, 10);
						else sprintf(output, "%10d", (int32)scores[rank].score);
						fwrite(output, sizeof(char), 10, out);
						continue;
					case 'G':
						// print a bar graph, 20 chars is a good length.
						if(rank >= listlen) 
						{
							memset(output, 32, 20);
							fwrite(output, sizeof(char), 20, out);
							continue;
						}

						// draw the foreground
						fwrite(hi_pi, sizeof(char), strlen(hi_pi), out);
						for(i = 0; i < (int32)scores[rank].percentage / 5; i++)
							fwrite(&hi_ch, sizeof(char), 1, out);
						
						// and the background
						fwrite(lo_pi, sizeof(char), strlen(lo_pi), out);
						for(; i < 20; i++)
							fwrite(&lo_ch, sizeof(char), 1, out);

						continue;
					}
				}
				else fwrite(buf, sizeof(char), 2 + numbytes, out);
			}
			else fwrite(buf, sizeof(char), 2, out);
		}
		else fwrite(buf, sizeof(char), 1, out);
	}
	fclose(in);
	fclose(out);
}

// i'm too lazy to #define offsets for everything, i'm going to magic number it for now.  fix later.
int dealwithconfig(struct cfgfile *cfg)
{
	int32 fsize, totalusers, listlen;
	int32 totalcalls, totalposts, totaluls, totaldls, totalulk, totaldlk;

	struct urec *users;
	struct userscore *scores;

	FILE *fp;

	char callstemplate[sizeof(cfg->templatepath) + sizeof(cfg->calltmpl)];
	char poststemplate[sizeof(cfg->templatepath) + sizeof(cfg->posttmpl)];
	char ultemplate[sizeof(cfg->templatepath) + sizeof(cfg->ultmpl)];
	char dltemplate[sizeof(cfg->templatepath) + sizeof(cfg->dltmpl)];
	char ulktemplate[sizeof(cfg->templatepath) + sizeof(cfg->ulktmpl)];
	char dlktemplate[sizeof(cfg->templatepath) + sizeof(cfg->dlktmpl)];
	
	char callsreport[sizeof(cfg->textpath) + sizeof(cfg->callrprt)];
	char postsreport[sizeof(cfg->textpath) + sizeof(cfg->postrprt)];
	char ulreport[sizeof(cfg->textpath) + sizeof(cfg->ulrprt)];
	char dlreport[sizeof(cfg->textpath) + sizeof(cfg->dlrprt)];
	char ulkreport[sizeof(cfg->textpath) + sizeof(cfg->ulkrprt)];
	char dlkreport[sizeof(cfg->textpath) + sizeof(cfg->dlkrprt)];


	sprintf(callstemplate,	"%s%s", cfg->templatepath, cfg->calltmpl);
	sprintf(poststemplate,	"%s%s", cfg->templatepath, cfg->posttmpl);
	sprintf(ultemplate,		"%s%s", cfg->templatepath, cfg->ultmpl);
	sprintf(dltemplate,		"%s%s", cfg->templatepath, cfg->dltmpl);
	sprintf(ulktemplate,	"%s%s", cfg->templatepath, cfg->ulktmpl);
	sprintf(dlktemplate,	"%s%s", cfg->templatepath, cfg->dlktmpl);
	
	sprintf(callsreport,	"%s%s", cfg->textpath, cfg->callrprt);
	sprintf(postsreport,	"%s%s", cfg->textpath, cfg->postrprt);
	sprintf(ulreport,		"%s%s", cfg->textpath, cfg->ulrprt);
	sprintf(dlreport,		"%s%s", cfg->textpath, cfg->dlrprt);
	sprintf(ulkreport,		"%s%s", cfg->textpath, cfg->ulkrprt);
	sprintf(dlkreport,		"%s%s", cfg->textpath, cfg->dlkrprt);

	listlen = cfg->listlength;
	
	totalcalls = totalposts = totaluls = totaldls = totalulk = totaldlk = 0;

	printf("\nprocessing %s\n", cfg->name);

	// calculate file size
	if(!(fp = fopen(cfg->usersdat, "rb"))) return 0;
	if(fseek(fp, 0, SEEK_END)) return 0;
	if((fsize = ftell(fp)) < 0) return 0;
	fclose(fp);

	// use filesize to calculate total number of users
	totalusers = fsize / UREC_SIZE;

	// fix a potential problem with small user databases and large lists
	listlen = min(totalusers, listlen);
	
	// load users from USERS.DAT
	users = (struct urec *)malloc(totalusers * sizeof(struct urec));

	if(!(fp = fopen(cfg->usersdat, "rb"))) return 0;
	for(int32 i = 0; i < totalusers; i++)
	{
		// load a user from the datafile
		load_urec(fp, users + i, i+1);

		// and update all the totals
		totalcalls += users[i].calls;
		totalposts += users[i].posts;
		totaluls   += users[i].ul;
		totaldls   += users[i].dl;
		totalulk   += users[i].ulk;
		totaldlk   += users[i].dlk;
	}
	fclose(fp);

	// generate reports, read templates and update the report ANSIs
	scores = (struct userscore *)malloc(listlen * sizeof(struct userscore));

	generateTopNScores(users, totalusers, scores, totalcalls, listlen, getCalls, cmpCalls);
	updateReport(callstemplate, callsreport, scores, listlen,
		cfg->hi_ch, cfg->hi_pi, cfg->lo_ch, cfg->lo_pi);

	generateTopNScores(users, totalusers, scores, totalposts, listlen, getPosts, cmpPosts);
	updateReport(poststemplate, postsreport, scores, listlen,
		cfg->hi_ch, cfg->hi_pi, cfg->lo_ch, cfg->lo_pi);

	generateTopNScores(users, totalusers, scores, totaluls, listlen, getULs, cmpULs);
	updateReport(ultemplate, ulreport, scores, listlen,
		cfg->hi_ch, cfg->hi_pi, cfg->lo_ch, cfg->lo_pi);

	generateTopNScores(users, totalusers, scores, totaldls, listlen, getDLs, cmpDLs);
	updateReport(dltemplate, dlreport, scores, listlen,
		cfg->hi_ch, cfg->hi_pi, cfg->lo_ch, cfg->lo_pi);

	generateTopNScores(users, totalusers, scores, totalulk, listlen, getULk, cmpULk);
	updateReport(ulktemplate, ulkreport, scores, listlen,
		cfg->hi_ch, cfg->hi_pi, cfg->lo_ch, cfg->lo_pi);

	generateTopNScores(users, totalusers, scores, totaldlk, listlen, getDLk, cmpDLk);
	updateReport(dlktemplate, dlkreport, scores, listlen,
		cfg->hi_ch, cfg->hi_pi, cfg->lo_ch, cfg->lo_pi);

	// free the dyncrap
	free(users);
	free(scores);
	
	return 1;
}

int main(int argc, char **argv)
{
	struct cfgfile cfg;

	// welcome to mtop.
	showversion();

	if(argc < 2)
	{
		showusage();
		return 1;
	}

	while(--argc)
	{
		load_cfg(&cfg, argv[argc]);
		dealwithconfig(&cfg);
	}

/*
	load_cfg(&cfg, "c:/mystic/mtop/default.cfg");
	dealwithconfig(&cfg);
*/
	
	return 0;
}
