#ifndef __MTOP_H
#define __MTOP_H

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "g_types.h"
#include "urec.h"

// i'm using this for temporary storage during report generation.
typedef struct userscore
{
	char handle[31];
	float64 score, percentage;
} userscore;


typedef struct cfgfile
{
	char name[32];
	int32	listlength;
	char	hi_ch, hi_pi[16], 
			lo_ch, lo_pi[16],
			usersdat[51],
			templatepath[41],
			textpath[41],
			calltmpl[32],
			posttmpl[32],
			ultmpl[32],
			dltmpl[32],
			ulktmpl[32],
			dlktmpl[32],
			callrprt[32],
			postrprt[32],
			ulrprt[32],
			dlrprt[32],
			ulkrprt[32],
			dlkrprt[32];
} cfgfile;

int load_cfg(struct cfgfile *cfg, const char *fn)
{
	FILE *fp;
	
	if(!(fp = fopen(fn, "r"))) return 0;

	if(fscanf(fp, "%d\n", &(cfg->listlength)) == EOF) return 0;
	if(fscanf(fp, "%c %s\n", &(cfg->hi_ch), cfg->hi_pi) == EOF) return 0;
	if(fscanf(fp, "%c %s\n", &(cfg->lo_ch), cfg->lo_pi) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->usersdat) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->templatepath) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->textpath) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->calltmpl) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->posttmpl) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->ultmpl) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->dltmpl) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->ulktmpl) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->dlktmpl) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->callrprt) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->postrprt) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->ulrprt) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->dlrprt) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->ulkrprt) == EOF) return 0;
	if(fscanf(fp, "%s\n", cfg->dlkrprt) == EOF) return 0;

	fclose(fp);

	strncpy(cfg->name, fn, sizeof(cfg->name));

	return 1;
}


// getData functions for generateTopNScores()
int32	getCalls	(struct urec *u)	{return u->calls;}
int32	getPosts	(struct urec *u)	{return u->posts;}
int32	getULs		(struct urec *u)	{return u->ul;}
int32	getDLs		(struct urec *u)	{return u->dl;}
int32	getULk		(struct urec *u)	{return u->ulk;}
int32	getDLk		(struct urec *u)	{return u->dlk;}

// cmp functions for qsort()
int cmpCalls	(const void *_u, const void *_v)
{
	struct urec *u = (struct urec *)_u;
	struct urec *v = (struct urec *)_v;
	return u->calls - v->calls;
}

int cmpPosts	(const void *_u, const void *_v)
{
	struct urec *u = (struct urec *)_u;
	struct urec *v = (struct urec *)_v;
	return u->posts - v->posts;
}

int cmpULs	(const void *_u, const void *_v)
{
	struct urec *u = (struct urec *)_u;
	struct urec *v = (struct urec *)_v;
	return u->ul - v->ul;
}

int cmpDLs	(const void *_u, const void *_v)
{
	struct urec *u = (struct urec *)_u;
	struct urec *v = (struct urec *)_v;
	return u->dl - v->dl;
}

int cmpULk	(const void *_u, const void *_v)
{
	struct urec *u = (struct urec *)_u;
	struct urec *v = (struct urec *)_v;
	return u->ulk - v->ulk;
}

int cmpDLk	(const void *_u, const void *_v)
{
	struct urec *u = (struct urec *)_u;
	struct urec *v = (struct urec *)_v;
	return u->dlk - v->dlk;
}



void showversion(void)
{
	printf("%s %s\n", "mtop -- Mystic BBS Top Ten Bulletin Generator -- version", MTOP_VERSION);
}

void showusage(void)
{
	printf("\nCreate reports:\n\tmtop file1.cfg [file2.cfg] [file3.cfg] ... [fileN.cfg]\n\n");
}


#endif
